use bevy_ecs::world::World;

use crate::core::{
    app::{DEPTH_IMAGE_ID, context::WindowContext},
    render::{
        resources::vulkan::{
            VulkanCommandBufferAllocator, VulkanComputeQueue, VulkanDescriptorSetAllocator,
            VulkanDevice, VulkanGraphicsQueue, VulkanInstance, VulkanMemoryAllocator,
            VulkanTransferQueue,
        },
        resources::{pipeline::PipelineLoader, texture::TextureLoader},
        vulkan_context::VulkanContext,
    },
};

pub fn extract_vulkan_ressources(world: &mut World, vulkan_context: &VulkanContext) {
    let vulkano_context = vulkan_context.vulkano_context();

    let vulkan_instance = vulkano_context.instance();
    let vulkan_device = vulkano_context.device();
    let vulkan_graphics_queue = vulkano_context.graphics_queue();
    let vulkan_compute_queue = vulkano_context.compute_queue();
    let vulkan_transfer_queue = vulkano_context.transfer_queue();
    let vulkan_memory_allocator = vulkano_context.memory_allocator();
    let vulkan_command_buffer_allocator = vulkan_context.command_buffer_allocator();
    let vulkan_descriptor_set_allocator = vulkan_context.descriptor_set_allocator();

    world.insert_resource(VulkanInstance::from(vulkan_instance));
    world.insert_resource(VulkanDevice::from(vulkan_device));
    world.insert_resource(VulkanGraphicsQueue::from(vulkan_graphics_queue));
    world.insert_resource(VulkanComputeQueue::from(vulkan_compute_queue));
    world.insert_resource(VulkanTransferQueue::from(vulkan_transfer_queue));
    world.insert_resource(VulkanMemoryAllocator::from(vulkan_memory_allocator));
    world.insert_resource(VulkanCommandBufferAllocator::from(
        vulkan_command_buffer_allocator,
    ));
    world.insert_resource(VulkanDescriptorSetAllocator::from(
        vulkan_descriptor_set_allocator,
    ));
}

pub fn extract_texture_loader(world: &mut World, vulkan_context: &VulkanContext) {
    let vulkano_context = vulkan_context.vulkano_context();

    let vulkan_device = vulkano_context.device();
    let vulkan_command_buffer_allocator = vulkan_context.command_buffer_allocator();
    let vulkan_memory_allocator = vulkano_context.memory_allocator();
    let vulkan_graphics_queue = vulkano_context.graphics_queue();
    let vulkan_transfer_queue = vulkano_context.transfer_queue();

    let texture_loader = TextureLoader::new(
        vulkan_device.clone(),
        vulkan_command_buffer_allocator.clone(),
        vulkan_memory_allocator.clone(),
        TextureLoader::select_best_suitable_queue(vulkan_graphics_queue, vulkan_transfer_queue),
    );

    world.insert_resource(texture_loader);
}

pub fn extract_pipeline_loader(world: &mut World, window_context: &mut WindowContext) {
    let vulkan_device = window_context
        .vulkan_context()
        .vulkano_context()
        .device()
        .clone();

    let depth_image_view = window_context
        .with_renderer_mut(|renderer| renderer.get_additional_image_view(DEPTH_IMAGE_ID).clone());

    let swapchain_image_view =
        window_context.with_renderer(|renderer| renderer.swapchain_image_view().clone());

    let pipeline_loader = PipelineLoader::new(
        vulkan_device.clone(),
        swapchain_image_view.format(),
        depth_image_view.format(),
    );

    world.insert_resource(pipeline_loader);
}
