use std::error::Error;

use crate::core::app::context::WindowContext;
use crate::core::input::InputManagerResource;
use crate::core::scene::AsScene;
use crate::core::scene::extract::{
    extract_pipeline_loader, extract_texture_loader, extract_vulkan_ressources,
};
use bevy_ecs::world::World;

use super::Scene;

#[derive(Default)]
pub struct SceneManager {
    current_scene: Option<Scene>,
    new_scene: Option<Box<dyn AsScene>>,
}

impl SceneManager {
    pub fn new() -> Self {
        Self {
            current_scene: None,
            new_scene: None,
        }
    }

    fn create_world_with_resources(window_context: &mut WindowContext) -> World {
        let mut world = World::new();

        extract_vulkan_ressources(&mut world, window_context.vulkan_context());
        extract_texture_loader(&mut world, window_context.vulkan_context());
        extract_pipeline_loader(&mut world, window_context);
        world.insert_resource(InputManagerResource(window_context.input_manager.clone()));

        world
    }

    pub fn set_new_scene(&mut self, scene_impl: Box<dyn AsScene>) {
        self.new_scene = Some(scene_impl);
    }

    pub fn current_scene(&self) -> Option<&Scene> {
        self.current_scene.as_ref()
    }

    pub fn current_scene_mut(&mut self) -> Option<&mut Scene> {
        self.current_scene.as_mut()
    }

    pub fn load_scene_if_not_loaded(
        &mut self,
        window_context: &mut WindowContext,
    ) -> Result<(), Box<dyn Error>> {
        if let Some(new_scene) = self.new_scene.take() {
            let world = Self::create_world_with_resources(window_context);
            let mut scene = Scene::new(new_scene, world);
            scene.load(window_context)?;

            if let Some(mut current_scene) = self.current_scene.take() {
                current_scene.unload();
            }

            self.current_scene = Some(scene);
        }
        Ok(())
    }
}
